package gov.usgs.earthquake.nshmp.model;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import org.junit.jupiter.api.Test;

import gov.usgs.earthquake.nshmp.NamedLocation;
import gov.usgs.earthquake.nshmp.gmm.Imt;
import gov.usgs.earthquake.nshmp.model.NshmTestUtils.Nshm;
import gov.usgs.earthquake.nshmp.model.NshmTestUtils.NshmInfo;
import gov.usgs.earthquake.nshmp.model.NshmTestUtils.NshmModel;
import gov.usgs.earthquake.nshmp.site.NshmpSite;

/**
 * Test NSHMs.
 */
class NshmTests {
  static Path DATA_PATH = Paths.get("src/test/resources/e2e/actual");

  /* Alaska test sites */
  private static final List<NamedLocation> ALASKA_LOCATIONS = List.of(
      NshmpSite.ANCHORAGE_AK,
      NshmpSite.FAIRBANKS_AK,
      NshmpSite.JUNEAU_AK,
      NshmpSite.KODIAK_AK,
      NshmpSite.VALDEZ_AK);

  /* CONUS test sites */
  private static final List<NamedLocation> CONUS_LOCATIONS = List.of(
      NshmpSite.LOS_ANGELES_CA,
      NshmpSite.SAN_FRANCISCO_CA,
      NshmpSite.SEATTLE_WA,
      NshmpSite.SALT_LAKE_CITY_UT,
      NshmpSite.RENO_NV,
      NshmpSite.NEW_MADRID_MO,
      NshmpSite.BOSTON_MA,
      NshmpSite.NEW_YORK_NY,
      NshmpSite.CHICAGO_IL);

  /* Hawaii test sites */
  private static final List<NamedLocation> HAWAII_LOCATIONS = List.of(
      NshmpSite.HILO_HI,
      NshmpSite.HONOLULU_HI,
      NshmpSite.KAILUA_KONA_HI);

  private static final Set<Imt> IMTS = EnumSet.of(Imt.PGA, Imt.SA0P2, Imt.SA1P0, Imt.SA5P0);
  private static final Set<Imt> AK_2007_IMTS = EnumSet.of(Imt.PGA, Imt.SA0P2, Imt.SA1P0);

  static final Map<String, Nshm> NSHMS;

  static {
    Map<String, Nshm> nshms = new HashMap<>();

    for (NshmInfo nshmInfo : NshmTestUtils.readNshms().nshms) {
      List<NamedLocation> locations = new ArrayList<>();
      Set<Imt> imts = IMTS;

      switch (nshmInfo.repo) {
        case "nshm-conus": {
          locations = CONUS_LOCATIONS;
          break;
        }
        case "nshm-alaska": {
          locations = ALASKA_LOCATIONS;
          if (nshmInfo.year == 2007) {
            imts = AK_2007_IMTS;
          }
          break;
        }
        case "nshm-hawaii": {
          locations = HAWAII_LOCATIONS;
          break;
        }
        default:
          throw new RuntimeException(nshmInfo.repo + " not supported");
      }

      nshms.put(nshmInfo.repo + "-" + nshmInfo.year, new Nshm(nshmInfo, locations, imts));
    }

    NSHMS = nshms;
  }

  /**
   * Generate results for all {@link NSHM}s
   *
   * Run "./gradlew nshms" to first download all NSHMs
   */
  public static void main(String[] args) throws IOException {
    /* Initialize and shut down executor to generate results. */

    // run single model
    NshmModel nshmModel =
        NshmTestUtils.loadModel(NSHMS.get("nshm-alaska-2023"));

    NshmTestUtils.writeExpecteds(nshmModel, Optional.empty());
    nshmModel.exec.shutdown();

    // run all models
    // for (Nshm nshm : NSHMS.values()) {
    // NshmModel nshmModel = NshmTestUtils.loadModel(nshm);
    // NshmTestUtils.writeExpecteds(nshmModel);
    // nshmModel.exec.shutdown();
    // }
  }

  /**
   * Test Alaska 2007 NSHM
   *
   * To run test: ./gradlew testAlaska2007
   */
  @Test
  final void testAlaska2007() throws IOException {
    Nshm nshm = NSHMS.get("nshm-alaska-2007");
    NshmTestUtils.testNshm(nshm, getDataPath(nshm));
  }

  /**
   * Test Alaska 2023 NSHM
   *
   * To run test: ./gradlew testAlaska2023
   */
  @Test
  final void testAlaska2023() throws IOException {
    Nshm nshm = NSHMS.get("nshm-alaska-2023");
    NshmTestUtils.testNshm(nshm, getDataPath(nshm));
  }

  /**
   * Test CONUS 2018 NSHM
   *
   * To run test: ./gradlew testConus2018
   */
  @Test
  final void testConus2018() throws IOException {
    Nshm nshm = NSHMS.get("nshm-conus-2018");
    NshmTestUtils.testNshm(nshm, getDataPath(nshm));
  }

  /**
   * Test CONUS 2023 NSHM
   *
   * To run test: ./gradlew testConus2023
   */
  @Test
  final void testConus2023() throws IOException {
    Nshm nshm = NSHMS.get("nshm-conus-2023");
    NshmTestUtils.testNshm(nshm, getDataPath(nshm));
  }

  /**
   * Test Hawaii 2021 NSHM
   *
   * To run test: ./gradlew testHawaii2021
   */
  @Test
  final void testHawaii2021() throws IOException {
    Nshm nshm = NSHMS.get("nshm-hawaii-2021");
    NshmTestUtils.testNshm(nshm, getDataPath(nshm));
  }

  private static Optional<Path> getDataPath(Nshm nshm) {
    return Files.exists(DATA_PATH.resolve(nshm.modelName())) ? Optional.of(DATA_PATH)
        : Optional.empty();
  }
}
